package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.*;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Array;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.state.Session;
import eu.jonahbauer.wizard.common.messages.client.CreateSessionMessage;
import eu.jonahbauer.wizard.common.messages.data.SessionData;
import eu.jonahbauer.wizard.common.messages.server.SessionJoinedMessage;
import eu.jonahbauer.wizard.common.model.Configuration;

public class CreateGameScreen extends MenuScreen {
    
    private TextButton buttonBack;
    private TextButton buttonContinue;

    private TextField sessionName;
    private TextField playerName;
    private TextField timeOut;

    long selectedTimeOut;
    String userName;
    SelectBox<String> configurations;

    private final ChangeListener listener = new ChangeListener() {
        @Override
        public void changed(ChangeEvent event, Actor actor) {
            if (actor == buttonBack) {
                game.setScreen(new LobbyScreen(game));
                sfxClick();
            } else if (actor == buttonContinue) {
                if( sessionName.getText() == null ) {
                    System.out.println("Please select a session name.");
                    sessionName.clear();
                }
                if (playerName.getText() == null) {
                    userName = "Mallory";
                    System.out.println("Your name will be " + userName + ".");
                }
                try{
                    selectedTimeOut = Long.parseLong(timeOut.getText());
                } catch (NumberFormatException e) {
                    System.out.println("Please select a valid timeout.");
                    timeOut.clear();
                }
                int selected = configurations.getSelectedIndex();
                Configuration config = Configuration.values()[selected];
                game.getClient().send(new CreateSessionMessage(sessionName.getText(), playerName.getText(), selectedTimeOut, config));
            }
        }
    };
    
    public CreateGameScreen(WizardGame game) {
        super(game);
    }
    
    @Override
    public void show() {
        super.show();

        buttonBack = new TextButton(game.messages.get("menu.connect.back"), game.data.skin);
        buttonBack.setPosition(WizardGame.WIDTH * 0.275f, BUTTON_BAR_Y);
        buttonContinue = new TextButton("Erstellen", game.data.skin);
        buttonContinue.setPosition(WizardGame.WIDTH * 0.725f - buttonContinue.getWidth(), BUTTON_BAR_Y);

        sessionName = new TextField("", game.data.skin);
        sessionName.setPosition(WizardGame.WIDTH * 0.3f, WizardGame.HEIGHT * 0.5f);
        sessionName.setSize(0.4f * WizardGame.WIDTH, 64);

        playerName = new TextField("", game.data.skin);
        playerName.setPosition(WizardGame.WIDTH * 0.3f, WizardGame.HEIGHT * 0.45f);
        playerName.setSize(0.4f * WizardGame.WIDTH, 64);

        timeOut = new TextField("", game.data.skin);
        timeOut.setPosition(WizardGame.WIDTH * 0.3f, WizardGame.HEIGHT * 0.4f);
        timeOut.setSize(0.4f * WizardGame.WIDTH, 64);
        
        configurations = new SelectBox<>(game.data.skin);
        configurations.setSize(400, 64);
        configurations.setPosition(WizardGame.WIDTH * 0.3f, WizardGame.HEIGHT * 0.3f);
        
        Array<String> values = new Array<>();
        for (Configuration value: Configuration.values() ) {
            values.add(value.toString());
        }

        configurations.setItems(values);
    
        var contentTable = new Table().center().left();
        contentTable.columnDefaults(0).growX().width(0.4f * WizardGame.WIDTH - 20);
        contentTable.setSize(0.4f * WizardGame.WIDTH - 20, 400);
        
        contentTable.add(new Label(game.messages.get("menu.lobby.player_name.label"), game.data.skin)).row();
        contentTable.add(playerName).row();
        contentTable.add(new Label(game.messages.get("menu.lobby.session_name.label"), game.data.skin)).row();
        contentTable.add(sessionName).row();
        contentTable.add(new Label(game.messages.get("menu.lobby.create_session.timeout.label"), game.data.skin)).row();
        contentTable.add(timeOut).row();
        contentTable.add(new Label(game.messages.get("menu.lobby.session_configuration.label"), game.data.skin)).row();
        contentTable.add(configurations).row();
        
        contentTable.setPosition(WizardGame.WIDTH * 0.3f, WizardGame.HEIGHT * 0.3f);
        
        
        
        Gdx.input.setInputProcessor(game.data.stage);
        game.data.stage.addActor(buttonContinue);
        game.data.stage.addActor(contentTable);
        game.data.stage.addActor(buttonBack);
        
        buttonContinue.addListener(listener);
        buttonBack.addListener(listener);
    }
    
    public void performTransition(SessionJoinedMessage joined) {
        int selected = configurations.getSelectedIndex();
        Configuration config = Configuration.values()[selected];
        System.out.println("Transition from Lobby to Session");
        SessionData sessionData = new SessionData(joined.getSession(), sessionName.getText(), joined.getPlayers().size(), config);
        game.getClient().transition(new Session(joined, sessionData, playerName.getText()));
    }
}
